# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/09_cell_id.ipynb

__all__ = ['exported_function']

# %% nbs/09_cell_id.ipynb 12
def exported_function():
    """This cell has both cell_id and export directives."""
    return "exported!"
