# AUTOGENERATED! DO NOT EDIT! Files to edit: nbs/02_export_to_1.ipynb, nbs/02_export_to_2.ipynb

__all__ = ['this_will_first_of_all', 'will_be_first', 'will_be_last']

# %% nbs/02_export_to_2.ipynb 2
def this_will_first_of_all():
    """Explicitly specifies the target module."""
    return "explicit"

# %% nbs/02_export_to_1.ipynb 3
def will_be_first():
    """Specifies module and order (lower = earlier in file)."""
    return "ordered"

# %% nbs/02_export_to_1.ipynb 2
def will_be_last():
    """Explicitly specifies the target module."""
    return "explicit"
